/*
 *  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#include <time.h>
#include "e.h"
#include "e_module.h"
#include "e_mod_main.h"
#include "CalcEphem.h"

//#define DEBUG

/* module private routines */
static Moon      *_moon_new(E_Module *module); 
static void       _moon_shutdown(Moon *moon);
static void       _moon_config_menu_new(Moon *moon);

static Moon_Face *_moon_face_new(E_Container *con, E_Module *module);
static void       _moon_face_free(Moon_Face *face);
static void       _moon_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change change);

static int        _moon_cb_check(void *data);
static double     _moon_phase_calc();

#ifdef DEBUG
static void       _moon_face_signal_cb(void *data, Evas_Object *o, const char *sig, const char *src);
#endif

static int _moon_count;

#ifdef DEBUG
static double _test_val;
#endif

/* public module routines. all modules must have these */
E_Module_Api e_modapi =
{
   E_MODULE_API_VERSION,
   "Moon"
};

void *
e_modapi_init(E_Module *module)
{
   Moon *moon;

   moon = _moon_new(module);
// TODO - don't need this yet
//   module->config_menu = moon->config_menu;
   return moon; 
}

int
e_modapi_shutdown(E_Module *module)
{  
   Moon *moon;

   if (module->config_menu)
     module->config_menu = NULL;

   moon = module->data;
   if (moon)
     _moon_shutdown(moon);

   return 1;
}

int
e_modapi_save(E_Module *module)
{  
   return 1;
}

int
e_modapi_info(E_Module *module)
{  
// TODO - make icon
/*
   char buf[4096];

   snprintf(buf, sizeof(buf), "%s/module_icon.png", e_module_dir_get(module));
   module->icon_file = strdup(buf);
*/
   return 1;
}

int
e_modapi_about(E_Module *module)
{
   /* 
    * FIXME - need to gettextise this properly 
    */
   e_module_dialog_show(_("Moon Clock Module"),
	                _("Display current phase of the moon ala e16 E-MoonClock"));
   return 1;
}

static Moon *
_moon_new(E_Module *module) 
{
   Moon *moon;
   Evas_List *mans, *cons;

   _moon_count = 0;
   moon = E_NEW(Moon, 1);
   if (!moon) return NULL;

   _moon_config_menu_new(moon);

   for (mans = e_manager_list(); mans; mans = mans->next) 
     {
	E_Manager *man;
	
	man = mans->data;
	for (cons = man->containers; cons; cons = cons->next)
	  {
	     E_Container *con;
	     Moon_Face *face;

	     con = cons->data;
	     face = _moon_face_new(con, module);
	     if (face)
	       {
		  moon->faces = evas_list_append(moon->faces, face);
	       }
	  }
     }

#ifndef DEBUG
   // TODO - make this timer value configurable
   moon->moon_check_timer = ecore_timer_add(60.0, _moon_cb_check, moon);
#else 
   moon->moon_check_timer = ecore_timer_add(0.05, _moon_cb_check, moon);
   _test_val =0.0;
#endif

   _moon_cb_check(moon);
   return moon;
}

static void
_moon_shutdown(Moon *moon)
{
   Evas_List *list;

   for (list = moon->faces; list; list = list->next)
     _moon_face_free(list->data);
   evas_list_free(moon->faces);

   e_object_del(E_OBJECT(moon->config_menu));

   ecore_timer_del(moon->moon_check_timer);
   
   E_FREE(moon);
}

static void
_moon_config_menu_new(Moon *moon)
{
   E_Menu_Item *mi;

   moon->config_menu = e_menu_new();
// TODO - use these
   mi = e_menu_item_new(moon->config_menu);
   e_menu_item_label_set(mi, _("Moon Config Menu Item 1"));
   mi = e_menu_item_new(moon->config_menu);
   e_menu_item_separator_set(mi, 1);
   mi = e_menu_item_new(moon->config_menu);
   e_menu_item_label_set(mi, _("Moon Config Menu Item 2"));
}

static Moon_Face *
_moon_face_new(E_Container *con, E_Module *module)
{
   Moon_Face *face;
   Evas_Object *o;
   Evas_Coord x, y, w, h;
   char edje_path[PATH_MAX];

   face = E_NEW(Moon_Face, 1);
   if (!face) return NULL;

   face->con = con;
   e_object_ref(E_OBJECT(con));

   evas_event_freeze(con->bg_evas);
   o = edje_object_add(con->bg_evas);
   face->moon_object = o;
#ifdef DEBUG
   edje_object_signal_callback_add(o, "*", "*", _moon_face_signal_cb, NULL);
#endif

   snprintf(edje_path, sizeof(edje_path), 
	 "%s/moon.edj", e_module_dir_get(module));
   edje_object_file_set(face->moon_object, edje_path, "moon/main");
   evas_object_show(face->moon_object);
   evas_object_resize(face->moon_object, 96, 96);
//   edje_object_calc_force(face->moon_object);
   edje_object_part_geometry_get(face->moon_object, "moon", &x, &y, &w, &h);
   face->inset.l = x;
   face->inset.r = 96 - (x + w);
   face->inset.t = y;
   face->inset.b = 96 - (y + h);

   face->gmc = e_gadman_client_new(con->gadman);
   e_gadman_client_domain_set(face->gmc, "module.moon", _moon_count++);
   e_gadman_client_policy_set(face->gmc,
	                      E_GADMAN_POLICY_ANYWHERE |
			      E_GADMAN_POLICY_HMOVE |
			      E_GADMAN_POLICY_VMOVE |
			      E_GADMAN_POLICY_HSIZE |
			      E_GADMAN_POLICY_VSIZE);

   /* Get interface min / max sizes and tell gadman about it */
   edje_object_size_min_get(face->moon_object, &w, &h);
   e_gadman_client_min_size_set(face->gmc, w, h);
   edje_object_size_max_get(face->moon_object, &w, &h);
   e_gadman_client_max_size_set(face->gmc, w, h);

   e_gadman_client_auto_size_set(face->gmc,
	                         64 + (face->inset.l + face->inset.r),
				 64 + (face->inset.t + face->inset.b));
   e_gadman_client_align_set(face->gmc, 1.0, 1.0);
   e_gadman_client_aspect_set(face->gmc, 1.0, 1.0);
   e_gadman_client_padding_set(face->gmc,
	                       face->inset.l, face->inset.r,
			       face->inset.t, face->inset.b);
   e_gadman_client_resize(face->gmc, 
	                  64 + (face->inset.l + face->inset.r),
			  64 + (face->inset.t + face->inset.b));
   e_gadman_client_change_func_set(face->gmc, _moon_face_cb_gmc_change, face);
   e_gadman_client_load(face->gmc);

   evas_event_thaw(con->bg_evas);

   return face;
}

static void
_moon_face_free(Moon_Face *face)
{
   e_object_unref(E_OBJECT(face->con));
   e_object_del(E_OBJECT(face->gmc));
   evas_object_del(face->moon_object);

   E_FREE(face);
   _moon_count--;
}

/* FIXME - just a straight copy from the Clock module here... */
static void
_moon_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change change)
{
   Moon_Face *face;
   Evas_Coord x, y, w, h;

   face = data;
   switch (change)
     {
      case E_GADMAN_CHANGE_MOVE_RESIZE:
	 e_gadman_client_geometry_get(face->gmc, &x, &y, &w, &h);
	 evas_object_move(face->moon_object, x, y);
	 evas_object_resize(face->moon_object, w, h);
	 break;
      case E_GADMAN_CHANGE_RAISE:
	 evas_object_raise(face->moon_object);
	 break;
      case E_GADMAN_CHANGE_EDGE:
      case E_GADMAN_CHANGE_ZONE:
         /* FIXME
          * Must we do something here?
	  */
	 break;
     }
}

/* Get the current moonphase and tell the edje about it */
static int
_moon_cb_check(void *data)
{
   Moon *moon;
   Evas_List *l;
   Moon_Face *face;
//   Edje_Message_Float msg;
   Edje_Message_String_Float_Set msg;
   char str[32];

   msg.count = 1;
//   msg.val = _moon_phase_calc();
   msg.val[0] = _moon_phase_calc();
   snprintf(str, sizeof(str), "%.2f%%", msg.val[0] * 100);
   msg.str = str;
#ifdef DEBUG
   msg.val[0] = _test_val;
   snprintf(str, sizeof(str), "%.2f%%", msg.val[0] * 100);
   _test_val = _test_val + (1.0 / 180.0);
   if (_test_val > 1.0) 
      _test_val = 0.0;
#endif
   moon = data;
   for (l = moon->faces; l; l = l->next) {
      face = l->data;
//      edje_object_message_send(face->moon_object, EDJE_MESSAGE_FLOAT, 1, &msg);
      edje_object_message_send(face->moon_object, EDJE_MESSAGE_STRING_FLOAT_SET, 1, &msg);
   }
   return 1;
}

static double
_moon_phase_calc()
{
   time_t current_time, date;
   struct tm *universal_time;
   double UT;
   CTrans c;
   
   current_time = time(NULL);
   if (current_time == -1) return 0; 

   /* Get Universal Time */
   universal_time = gmtime(&current_time);

   /* Date in YYYYMMDD format */
   date = (universal_time->tm_year + 1900) * 10000 + 
          (universal_time->tm_mon + 1) * 100 +
	  universal_time->tm_mday;

   /* UTC representation in decimal hours */ 
   UT = universal_time->tm_hour +
        universal_time->tm_min / 60.0 +
        universal_time->tm_sec / 3600.0;

   CalcEphem(date, UT, &c);
#ifdef DEBUG
   printf("date is %f\n", (double) date);
   printf("UT is %f\n", (double) UT);
   printf("MoonPhase is %f\n", c.MoonPhase);
   printf("MoonAge is %f\n", c.MoonAge);
   printf("Visible is %d\n", c.Visible);
#endif

   if (c.MoonPhase < 0.0) 
      return 0.0;
   if (c.MoonPhase > 1.0) 
      return 1.0;
   return c.MoonPhase; 
}

#if DEBUG
void
_moon_face_signal_cb(void *data, Evas_Object *o, const char *sig, const char *src)
{
   if (!strcmp(sig, "MSG_FLOAT"))
     {
	printf("EMIT: \"%s\" \"%s\"\n", sig, src);
     }
   return;
}
#endif

